@@ -856,7 +856,7 @@ g_irepository_get_shared_library (GIRepository *repository,
 /**
  * g_irepository_get_c_prefix
  * @repository: A #GIRepository, may be %NULL for the default
- * @namespace: Namespace to inspect
+ * @namespace_: Namespace to inspect
  *
  * This function returns the "C prefix", or the C level namespace
  * associated with the given introspection namespace.  Each C symbol
@@ -1043,7 +1043,7 @@ free_candidate (struct NamespaceVersionCandidadate *candidate)
   g_mapped_file_unref (candidate->mfile);
   g_free (candidate->path);
   g_free (candidate->version);
-  g_free (candidate);
+  g_slice_free (struct NamespaceVersionCandidadate, candidate);
 }
 
 static GMappedFile *
@@ -1111,7 +1111,7 @@ find_namespace_latest (const gchar  *namespace,
 	      g_clear_error (&error);
 	      continue;
 	    }
-	  candidate = g_new0 (struct NamespaceVersionCandidadate, 1);
+	  candidate = g_slice_new0 (struct NamespaceVersionCandidadate);
 	  candidate->mfile = mfile;
 	  candidate->path_index = index;
 	  candidate->path = path;
@@ -1134,7 +1134,7 @@ find_namespace_latest (const gchar  *namespace,
       result = elected->mfile;
       *path_ret = elected->path;
       *version_ret = elected->version;
-      g_free (elected); /* just free the container */
+      g_slice_free (struct NamespaceVersionCandidadate, elected); /* just free the container */
       g_slist_foreach (candidates, (GFunc) free_candidate, NULL);
       g_slist_free (candidates);
     }
@@ -1149,7 +1149,7 @@ find_namespace_latest (const gchar  *namespace,
  * @repository: (allow-none): Repository, may be %NULL for the default
  * @namespace_: GI namespace to use, e.g. "Gtk"
  * @version: (allow-none): Version of namespace, may be %NULL for latest
- * @flags: Set of %GIRepositoryLoadFlags, may be %0
+ * @flags: Set of %GIRepositoryLoadFlags, may be 0
  * @error: a #GError.
  *
  * Force the namespace @namespace_ to be loaded if it isn't already.
