@@ -53,6 +53,13 @@ _g_irnode_dump_stats (void)
   g_message ("%lu types (%lu before sharing)", unique_types_count, types_count);
 }
 
+#define DO_ALIGNED_COPY(dest_addr, value, type) \
+do {                                            \
+	type tmp_var;	                        \
+	tmp_var = value;		        \
+	memcpy(dest_addr, &tmp_var, sizeof(type));	\
+} while(0)
+
 #define ALIGN_VALUE(this, boundary) \
   (( ((unsigned long)(this)) + (((unsigned long)(boundary)) -1)) & (~(((unsigned long)(boundary))-1)))
 
@@ -206,7 +213,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_CALLBACK:
       {
 	GIrNodeFunction *function = (GIrNodeFunction *)node;
-	
+
 	g_free (node->name);
 	g_free (function->symbol);
 	g_ir_node_free ((GIrNode *)function->result);
@@ -219,7 +226,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_TYPE:
       {
 	GIrNodeType *type = (GIrNodeType *)node;
-	
+
 	g_free (node->name);
 	g_ir_node_free ((GIrNode *)type->parameter_type1);
 	g_ir_node_free ((GIrNode *)type->parameter_type2);
@@ -233,7 +240,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_PARAM:
       {
 	GIrNodeParam *param = (GIrNodeParam *)node;
-	
+
 	g_free (node->name);
 	g_ir_node_free ((GIrNode *)param->type);
       }
@@ -242,7 +249,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_PROPERTY:
       {
 	GIrNodeProperty *property = (GIrNodeProperty *)node;
-	
+
 	g_free (node->name);
 	g_ir_node_free ((GIrNode *)property->type);
       }
@@ -251,7 +258,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_SIGNAL:
       {
 	GIrNodeSignal *signal = (GIrNodeSignal *)node;
-	
+
 	g_free (node->name);
 	for (l = signal->parameters; l; l = l->next)
 	  g_ir_node_free ((GIrNode *)l->data);
@@ -263,7 +270,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_VFUNC:
       {
 	GIrNodeVFunc *vfunc = (GIrNodeVFunc *)node;
-	
+
 	g_free (node->name);
 	g_free (vfunc->invoker);
 	for (l = vfunc->parameters; l; l = l->next)
@@ -276,9 +283,10 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_FIELD:
       {
 	GIrNodeField *field = (GIrNodeField *)node;
-	
+
 	g_free (node->name);
 	g_ir_node_free ((GIrNode *)field->type);
+	g_ir_node_free ((GIrNode *)field->callback);
       }
       break;
 
@@ -286,7 +294,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_INTERFACE:
       {
 	GIrNodeInterface *iface = (GIrNodeInterface *)node;
-	
+
 	g_free (node->name);
 	g_free (iface->gtype_name);
 	g_free (iface->gtype_init);
@@ -305,7 +313,7 @@ g_ir_node_free (GIrNode *node)
 
       }
       break;
- 
+
     case G_IR_NODE_VALUE:
       {
 	g_free (node->name);
@@ -316,7 +324,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_FLAGS:
       {
 	GIrNodeEnum *enum_ = (GIrNodeEnum *)node;
-	
+
 	g_free (node->name);
 	g_free (enum_->gtype_name);
 	g_free (enum_->gtype_init);
@@ -330,7 +338,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_BOXED:
       {
 	GIrNodeBoxed *boxed = (GIrNodeBoxed *)node;
-	
+
 	g_free (node->name);
 	g_free (boxed->gtype_name);
 	g_free (boxed->gtype_init);
@@ -358,7 +366,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_CONSTANT:
       {
 	GIrNodeConstant *constant = (GIrNodeConstant *)node;
-	
+
 	g_free (node->name);
 	g_free (constant->value);
 	g_ir_node_free ((GIrNode *)constant->type);
@@ -368,7 +376,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_ERROR_DOMAIN:
       {
 	GIrNodeErrorDomain *domain = (GIrNodeErrorDomain *)node;
-	
+
 	g_free (node->name);
 	g_free (domain->getquark);
 	g_free (domain->codes);
@@ -378,7 +386,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_XREF:
       {
 	GIrNodeXRef *xref = (GIrNodeXRef *)node;
-	
+
 	g_free (node->name);
 	g_free (xref->namespace);
       }
@@ -387,7 +395,7 @@ g_ir_node_free (GIrNode *node)
     case G_IR_NODE_UNION:
       {
 	GIrNodeUnion *union_ = (GIrNodeUnion *)node;
-	
+
 	g_free (node->name);
 	g_free (union_->gtype_name);
 	g_free (union_->gtype_init);
@@ -403,7 +411,7 @@ g_ir_node_free (GIrNode *node)
     default:
       g_error ("Unhandled node type %d\n", node->type);
       break;
-    } 
+    }
 
   g_hash_table_destroy (node->attributes);
 
@@ -464,7 +472,7 @@ g_ir_node_get_size (GIrNode *node)
     case G_IR_NODE_FLAGS:
       {
 	GIrNodeEnum *enum_ = (GIrNodeEnum *)node;
-	
+
 	size = sizeof (EnumBlob);
 	for (l = enum_->values; l; l = l->next)
 	  size += g_ir_node_get_size ((GIrNode *)l->data);
@@ -508,7 +516,13 @@ g_ir_node_get_size (GIrNode *node)
       break;
 
     case G_IR_NODE_FIELD:
-      size = sizeof (FieldBlob);
+      {
+	GIrNodeField *field = (GIrNodeField *)node;
+
+        size = sizeof (FieldBlob);
+        if (field->callback)
+          size += g_ir_node_get_size ((GIrNode *)field->callback);
+      }
       break;
 
     case G_IR_NODE_CONSTANT:
@@ -535,7 +549,7 @@ g_ir_node_get_size (GIrNode *node)
       }
       break;
 
-    default: 
+    default:
       g_error ("Unhandled node type '%s'\n",
 	       g_ir_node_type_to_string (node->type));
       size = 0;
@@ -603,12 +617,12 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
     case G_IR_NODE_PARAM:
       {
 	GIrNodeParam *param = (GIrNodeParam *)node;
-	
+
 	/* See the comment in the G_IR_NODE_PARAM/ArgBlob writing below */
 	size = sizeof (ArgBlob) - sizeof (SimpleTypeBlob);
 	if (node->name)
 	  size += ALIGN_VALUE (strlen (node->name) + 1, 4);
-	size += g_ir_node_get_full_size_internal (node, (GIrNode *)param->type);	
+	size += g_ir_node_get_full_size_internal (node, (GIrNode *)param->type);
       }
       break;
 
@@ -647,7 +661,7 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
 	      case GI_TYPE_TAG_ERROR:
 		{
 		  gint n;
-		  
+
 		  if (type->errors)
 		    n = g_strv_length (type->errors);
 		  else
@@ -705,7 +719,7 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
     case G_IR_NODE_FLAGS:
       {
 	GIrNodeEnum *enum_ = (GIrNodeEnum *)node;
-	
+
 	size = sizeof (EnumBlob);
 	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
 	if (enum_->gtype_name)
@@ -715,7 +729,7 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
 	  }
 
 	for (l = enum_->values; l; l = l->next)
-	  size += g_ir_node_get_full_size_internal (node, (GIrNode *)l->data);	
+	  size += g_ir_node_get_full_size_internal (node, (GIrNode *)l->data);
       }
       break;
 
@@ -760,10 +774,10 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
     case G_IR_NODE_PROPERTY:
       {
 	GIrNodeProperty *prop = (GIrNodeProperty *)node;
-	
+
 	size = sizeof (PropertyBlob);
 	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
-	size += g_ir_node_get_full_size_internal (node, (GIrNode *)prop->type);	
+	size += g_ir_node_get_full_size_internal (node, (GIrNode *)prop->type);
       }
       break;
 
@@ -797,7 +811,10 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
 
 	size = sizeof (FieldBlob);
 	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
-	size += g_ir_node_get_full_size_internal (node, (GIrNode *)field->type);	
+	if (field->callback)
+          size += g_ir_node_get_full_size_internal (node, (GIrNode *)field->callback);
+	else
+          size += g_ir_node_get_full_size_internal (node, (GIrNode *)field->type);
       }
       break;
 
@@ -809,7 +826,7 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
 	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
 	/* FIXME non-string values */
 	size += ALIGN_VALUE (strlen (constant->value) + 1, 4);
-	size += g_ir_node_get_full_size_internal (node, (GIrNode *)constant->type);	
+	size += g_ir_node_get_full_size_internal (node, (GIrNode *)constant->type);
       }
       break;
 
@@ -826,7 +843,7 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
     case G_IR_NODE_XREF:
       {
 	GIrNodeXRef *xref = (GIrNodeXRef *)node;
-	
+
 	size = 0;
 	size += ALIGN_VALUE (strlen (node->name) + 1, 4);
 	size += ALIGN_VALUE (strlen (xref->namespace) + 1, 4);
@@ -850,7 +867,7 @@ g_ir_node_get_full_size_internal (GIrNode *parent,
       }
       break;
 
-    default: 
+    default:
       g_error ("Unknown type tag %d\n", node->type);
       size = 0;
     }
@@ -930,7 +947,7 @@ g_ir_node_add_member (GIrNode         *node,
 {
   g_return_if_fail (node != NULL);
   g_return_if_fail (member != NULL);
-		    
+
   switch (node->type)
     {
     case G_IR_NODE_OBJECT:
@@ -1009,7 +1026,7 @@ parse_boolean_value (const gchar *str)
 {
   if (strcmp (str, "TRUE") == 0)
     return TRUE;
-  
+
   if (strcmp (str, "FALSE") == 0)
     return FALSE;
 
@@ -1031,31 +1048,31 @@ find_entry_node (GIrModule   *module,
 
   g_assert (name != NULL);
   g_assert (strlen (name) > 0);
-  
+
   names = g_strsplit (name, ".", 0);
   n_names = g_strv_length (names);
   if (n_names > 2)
     g_error ("Too many name parts");
-  
+
   for (l = module->entries, i = 1; l; l = l->next, i++)
     {
       GIrNode *node = (GIrNode *)l->data;
-      
+
       if (n_names > 1)
 	{
 	  if (node->type != G_IR_NODE_XREF)
 	    continue;
-	  
+
 	  if (((GIrNodeXRef *)node)->namespace == NULL ||
 	      strcmp (((GIrNodeXRef *)node)->namespace, names[0]) != 0)
 	    continue;
 	}
-	 
+
       if (strcmp (node->name, names[n_names - 1]) == 0)
 	{
 	  if (idx)
 	    *idx = i;
-	  
+
 	  result = node;
 	  goto out;
 	}
@@ -1067,9 +1084,9 @@ find_entry_node (GIrModule   *module,
 
       ((GIrNodeXRef *)node)->namespace = g_strdup (names[0]);
       node->name = g_strdup (names[1]);
-  
+
       module->entries = g_list_append (module->entries, node);
-  
+
       if (idx)
 	*idx = g_list_length (module->entries);
 
@@ -1080,7 +1097,7 @@ find_entry_node (GIrModule   *module,
       goto out;
     }
 
-  g_warning ("Entry '%s' not found", name);
+  g_ir_module_fatal (module, 0, "Type reference '%s' not found", name);
 
  out:
 
@@ -1212,23 +1229,23 @@ get_index_of_member_type (GIrNodeInterface *node,
 }
 
 static void
-serialize_type (GIrModule    *module, 
+serialize_type (GIrModule    *module,
 		GList        *modules,
-		GIrNodeType  *node, 
+		GIrNodeType  *node,
 		GString      *str)
 {
   gint i;
   const gchar* basic[] = {
-    "void", 
-    "boolean", 
-    "int8", 
-    "uint8", 
-    "int16", 
-    "uint16", 
-    "int32", 
-    "uint32", 
-    "int64", 
-    "uint64", 
+    "void",
+    "boolean",
+    "int8",
+    "uint8",
+    "int16",
+    "uint16",
+    "int32",
+    "uint32",
+    "int64",
+    "uint64",
     "short",
     "ushort",
     "int",
@@ -1237,14 +1254,14 @@ serialize_type (GIrModule    *module,
     "ulong",
     "ssize",
     "size",
-    "float", 
+    "float",
     "double",
     "time_t",
     "GType",
-    "utf8", 
+    "utf8",
     "filename",
   };
-  
+
   if (node->tag < GI_TYPE_TAG_ARRAY)
     {
       g_string_append_printf (str, "%s%s", basic[node->tag],
@@ -1259,11 +1276,11 @@ serialize_type (GIrModule    *module,
 	g_string_append_printf (str, "length=%d", node->length);
       else if (node->has_size)
         g_string_append_printf (str, "fixed-size=%d", node->size);
-      
+
       if (node->zero_terminated)
-	g_string_append_printf (str, "%szero-terminated=1", 
+	g_string_append_printf (str, "%szero-terminated=1",
 				node->has_length ? "," : "");
-      
+
       g_string_append (str, "]");
     }
   else if (node->tag == GI_TYPE_TAG_INTERFACE)
@@ -1292,9 +1309,9 @@ serialize_type (GIrModule    *module,
       g_string_append (str, "GList");
       if (node->parameter_type1)
 	{
-	  g_string_append (str, "<"); 
+	  g_string_append (str, "<");
 	  serialize_type (module, modules, node->parameter_type1, str);
-	  g_string_append (str, ">"); 
+	  g_string_append (str, ">");
 	}
     }
   else if (node->tag == GI_TYPE_TAG_GSLIST)
@@ -1302,9 +1319,9 @@ serialize_type (GIrModule    *module,
       g_string_append (str, "GSList");
       if (node->parameter_type1)
 	{
-	  g_string_append (str, "<"); 
+	  g_string_append (str, "<");
 	  serialize_type (module, modules, node->parameter_type1, str);
-	  g_string_append (str, ">"); 
+	  g_string_append (str, ">");
 	}
     }
   else if (node->tag == GI_TYPE_TAG_GHASH)
@@ -1312,11 +1329,11 @@ serialize_type (GIrModule    *module,
       g_string_append (str, "GHashTable<");
       if (node->parameter_type1)
 	{
-	  g_string_append (str, "<"); 
+	  g_string_append (str, "<");
 	  serialize_type (module, modules, node->parameter_type1, str);
-	  g_string_append (str, ","); 
+	  g_string_append (str, ",");
 	  serialize_type (module, modules, node->parameter_type2, str);
-	  g_string_append (str, ">"); 
+	  g_string_append (str, ">");
 	}
     }
   else if (node->tag == GI_TYPE_TAG_ERROR)
@@ -1324,14 +1341,14 @@ serialize_type (GIrModule    *module,
       g_string_append (str, "GError");
       if (node->errors)
 	{
-	  g_string_append (str, "<"); 
+	  g_string_append (str, "<");
 	  for (i = 0; node->errors[i]; i++)
 	    {
 	      if (i > 0)
 		g_string_append (str, ",");
 	      g_string_append (str, node->errors[i]);
 	    }
-	  g_string_append (str, ">"); 
+	  g_string_append (str, ">");
 	}
     }
 }
@@ -1433,27 +1450,27 @@ g_ir_node_build_typelib (GIrNode         *node,
 	SimpleTypeBlob *blob = (SimpleTypeBlob *)&data[*offset];
 
 	*offset += sizeof (SimpleTypeBlob);
-	
+
 	if (type->tag < GI_TYPE_TAG_ARRAY ||
 	    type->tag == GI_TYPE_TAG_UTF8 ||
 	    type->tag == GI_TYPE_TAG_FILENAME)
-	  { 
+	  {
 	    blob->flags.reserved = 0;
 	    blob->flags.reserved2 = 0;
 	    blob->flags.pointer = type->is_pointer;
 	    blob->flags.reserved3 = 0;
 	    blob->flags.tag = type->tag;
 	  }
-	else 
+	else
 	  {
 	    GString *str;
 	    gchar *s;
 	    gpointer value;
-	    
+
 	    str = g_string_new (0);
 	    serialize_type (module, modules, type, str);
 	    s = g_string_free (str, FALSE);
-	    
+
 	    types_count += 1;
 	    value = g_hash_table_lookup (types, s);
 	    if (value)
@@ -1465,7 +1482,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	      {
 		unique_types_count += 1;
 		g_hash_table_insert (types, s, GUINT_TO_POINTER(*offset2));
-				     
+
 		blob->offset = *offset2;
 		switch (type->tag)
 		  {
@@ -1473,13 +1490,14 @@ g_ir_node_build_typelib (GIrNode         *node,
 		    {
 		      ArrayTypeBlob *array = (ArrayTypeBlob *)&data[*offset2];
 		      guint32 pos;
-		      
+
 		      array->pointer = 1;
 		      array->reserved = 0;
 		      array->tag = type->tag;
 		      array->zero_terminated = type->zero_terminated;
 		      array->has_length = type->has_length;
                       array->has_size = type->has_size;
+                      array->array_type = type->array_type;
 		      array->reserved2 = 0;
                       if (array->has_length)
                         array->dimensions.length = type->length;
@@ -1487,15 +1505,15 @@ g_ir_node_build_typelib (GIrNode         *node,
                         array->dimensions.size  = type->size;
                       else
                         array->dimensions.length = -1;
-		      
+
 		      pos = *offset2 + G_STRUCT_OFFSET (ArrayTypeBlob, type);
 		      *offset2 += sizeof (ArrayTypeBlob);
-		      
+
 		      g_ir_node_build_typelib ((GIrNode *)type->parameter_type1,
 		                               node, build, &pos, offset2);
 		    }
 		    break;
-		    
+
 		  case GI_TYPE_TAG_INTERFACE:
 		    {
 		      InterfaceTypeBlob *iface = (InterfaceTypeBlob *)&data[*offset2];
@@ -1509,69 +1527,69 @@ g_ir_node_build_typelib (GIrNode         *node,
 
 		    }
 		    break;
-		    
+
 		  case GI_TYPE_TAG_GLIST:
 		  case GI_TYPE_TAG_GSLIST:
 		    {
 		      ParamTypeBlob *param = (ParamTypeBlob *)&data[*offset2];
 		      guint32 pos;
-		      
+
 		      param->pointer = 1;
 		      param->reserved = 0;
 		      param->tag = type->tag;
 		      param->reserved2 = 0;
 		      param->n_types = 1;
-		      
+
 		      pos = *offset2 + G_STRUCT_OFFSET (ParamTypeBlob, type);
 		      *offset2 += sizeof (ParamTypeBlob) + sizeof (SimpleTypeBlob);
-		      
-		      g_ir_node_build_typelib ((GIrNode *)type->parameter_type1, 
+
+		      g_ir_node_build_typelib ((GIrNode *)type->parameter_type1,
 					       node, build, &pos, offset2);
 		    }
 		    break;
-		    
+
 		  case GI_TYPE_TAG_GHASH:
 		    {
 		      ParamTypeBlob *param = (ParamTypeBlob *)&data[*offset2];
 		      guint32 pos;
-		      
+
 		      param->pointer = 1;
 		      param->reserved = 0;
 		      param->tag = type->tag;
 		      param->reserved2 = 0;
 		      param->n_types = 2;
-		      
+
 		      pos = *offset2 + G_STRUCT_OFFSET (ParamTypeBlob, type);
 		      *offset2 += sizeof (ParamTypeBlob) + sizeof (SimpleTypeBlob)*2;
-		      
-		      g_ir_node_build_typelib ((GIrNode *)type->parameter_type1, 
+
+		      g_ir_node_build_typelib ((GIrNode *)type->parameter_type1,
 					       node, build, &pos, offset2);
-		      g_ir_node_build_typelib ((GIrNode *)type->parameter_type2, 
+		      g_ir_node_build_typelib ((GIrNode *)type->parameter_type2,
 					       node, build, &pos, offset2);
 		    }
 		    break;
-		    
+
 		  case GI_TYPE_TAG_ERROR:
 		    {
 		      ErrorTypeBlob *blob = (ErrorTypeBlob *)&data[*offset2];
 		      gint i;
-		      
+
 		      blob->pointer = 1;
 		      blob->reserved = 0;
 		      blob->tag = type->tag;
 		      blob->reserved2 = 0;
-		      if (type->errors) 
+		      if (type->errors)
 			blob->n_domains = g_strv_length (type->errors);
 		      else
 			blob->n_domains = 0;
-		      
+
 		      *offset2 = ALIGN_VALUE (*offset2 + G_STRUCT_OFFSET (ErrorTypeBlob, domains)
 		                              + 2 * blob->n_domains, 4);
 		      for (i = 0; i < blob->n_domains; i++)
 			blob->domains[i] = find_entry (module, modules, type->errors[i]);
 		    }
 		    break;
-		    
+
 		  default:
 		    g_error ("Unknown type tag %d\n", type->tag);
 		    break;
@@ -1587,8 +1605,6 @@ g_ir_node_build_typelib (GIrNode         *node,
 	FieldBlob *blob;
 
 	blob = (FieldBlob *)&data[*offset];
-	/* We handle the size member specially below, so subtract it */
-	*offset += sizeof (FieldBlob) - sizeof (SimpleTypeBlob);
 
 	blob->name = write_string (node->name, strings, data, offset2);
 	blob->readable = field->readable;
@@ -1600,8 +1616,22 @@ g_ir_node_build_typelib (GIrNode         *node,
 	else
 	  blob->struct_offset = 0xFFFF; /* mark as unknown */
 
-        g_ir_node_build_typelib ((GIrNode *)field->type, 
-				 node, build, offset, offset2);
+        if (field->callback)
+          {
+            blob->has_embedded_type = TRUE;
+            blob->type.offset = GI_INFO_TYPE_CALLBACK;
+	    *offset += sizeof (FieldBlob);
+            g_ir_node_build_typelib ((GIrNode *)field->callback,
+			             node, build, offset, offset2);
+          }
+        else
+          {
+            blob->has_embedded_type = FALSE;
+            /* We handle the size member specially below, so subtract it */
+	    *offset += sizeof (FieldBlob) - sizeof (SimpleTypeBlob);
+            g_ir_node_build_typelib ((GIrNode *)field->type,
+				     node, build, offset, offset2);
+          }
       }
       break;
 
@@ -1615,12 +1645,12 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->name = write_string (node->name, strings, data, offset2);
 	blob->deprecated = prop->deprecated;
 	blob->readable = prop->readable;
- 	blob->writable = prop->writable;
- 	blob->construct = prop->construct;
- 	blob->construct_only = prop->construct_only;
+	blob->writable = prop->writable;
+	blob->construct = prop->construct;
+	blob->construct_only = prop->construct_only;
 	blob->reserved = 0;
 
-        g_ir_node_build_typelib ((GIrNode *)prop->type, 
+        g_ir_node_build_typelib ((GIrNode *)prop->type,
 				 node, build, offset, offset2);
       }
       break;
@@ -1654,7 +1684,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 
 	g_debug ("building function '%s'", function->symbol);
 
-        g_ir_node_build_typelib ((GIrNode *)function->result->type, 
+        g_ir_node_build_typelib ((GIrNode *)function->result->type,
 				 node, build, &signature, offset2);
 
 	blob2->may_return_null = function->result->allow_none;
@@ -1664,7 +1694,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob2->n_arguments = n;
 
 	signature += 4;
-	
+
 	for (l = function->parameters; l; l = l->next)
 	  {
 	    GIrNode *param = (GIrNode *)l->data;
@@ -1694,8 +1724,8 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->reserved = 0;
 	blob->name = write_string (node->name, strings, data, offset2);
 	blob->signature = signature;
-	
-        g_ir_node_build_typelib ((GIrNode *)function->result->type, 
+
+        g_ir_node_build_typelib ((GIrNode *)function->result->type,
 				 node, build, &signature, offset2);
 
 	blob2->may_return_null = function->result->allow_none;
@@ -1705,7 +1735,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob2->n_arguments = n;
 
 	signature += 4;
-	
+
 	for (l = function->parameters; l; l = l->next)
 	  {
 	    GIrNode *param = (GIrNode *)l->data;
@@ -1743,8 +1773,8 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->class_closure = 0; /* FIXME */
 	blob->name = write_string (node->name, strings, data, offset2);
 	blob->signature = signature;
-	
-        g_ir_node_build_typelib ((GIrNode *)signal->result->type, 
+
+        g_ir_node_build_typelib ((GIrNode *)signal->result->type,
 				 node, build, &signature, offset2);
 
 	blob2->may_return_null = signal->result->allow_none;
@@ -1754,7 +1784,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob2->n_arguments = n;
 
 	signature += 4;
-	
+
 	for (l = signal->parameters; l; l = l->next)
 	  {
 	    GIrNode *param = (GIrNode *)l->data;
@@ -1800,8 +1830,8 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->struct_offset = vfunc->offset;
 	blob->reserved2 = 0;
 	blob->signature = signature;
-	
-        g_ir_node_build_typelib ((GIrNode *)vfunc->result->type, 
+
+        g_ir_node_build_typelib ((GIrNode *)vfunc->result->type,
 				 node, build, &signature, offset2);
 
 	blob2->may_return_null = vfunc->result->allow_none;
@@ -1811,7 +1841,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob2->n_arguments = n;
 
 	signature += 4;
-	
+
 	for (l = vfunc->parameters; l; l = l->next)
 	  {
 	    GIrNode *param = (GIrNode *)l->data;
@@ -1831,10 +1861,10 @@ g_ir_node_build_typelib (GIrNode         *node,
 	 */
 	*offset += sizeof (ArgBlob) - sizeof (SimpleTypeBlob);
 
- 	blob->name = write_string (node->name, strings, data, offset2);
- 	blob->in = param->in;
- 	blob->out = param->out;
- 	blob->dipper = param->dipper;
+	blob->name = write_string (node->name, strings, data, offset2);
+	blob->in = param->in;
+	blob->out = param->out;
+	blob->dipper = param->dipper;
 	blob->allow_none = param->allow_none;
 	blob->optional = param->optional;
 	blob->transfer_ownership = param->transfer;
@@ -1844,7 +1874,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->reserved = 0;
         blob->closure = param->closure;
         blob->destroy = param->destroy;
-        
+
         g_ir_node_build_typelib ((GIrNode *)param->type, node, build, offset, offset2);
       }
       break;
@@ -1854,8 +1884,9 @@ g_ir_node_build_typelib (GIrNode         *node,
 	StructBlob *blob = (StructBlob *)&data[*offset];
 	GIrNodeStruct *struct_ = (GIrNodeStruct *)node;
 	GList *members;
-	
+
 	blob->blob_type = BLOB_TYPE_STRUCT;
+        blob->foreign = struct_->foreign;
 	blob->deprecated = struct_->deprecated;
 	blob->is_gtype_struct = struct_->is_gtype_struct;
 	blob->reserved = 0;
@@ -1938,7 +1969,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 
 	blob->blob_type = BLOB_TYPE_UNION;
 	blob->deprecated = union_->deprecated;
- 	blob->reserved = 0;
+	blob->reserved = 0;
 	blob->name = write_string (node->name, strings, data, offset2);
 	blob->alignment = union_->alignment;
 	blob->size = union_->size;
@@ -1966,7 +1997,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	  {
 	    *offset += 28;
 	    blob->discriminated = TRUE;
-	    g_ir_node_build_typelib ((GIrNode *)union_->discriminator_type, 
+	    g_ir_node_build_typelib ((GIrNode *)union_->discriminator_type,
 				     build, offset, offset2);
 	  }
 	else
@@ -1975,7 +2006,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	*offset += sizeof (UnionBlob);
 	blob->discriminated = FALSE;
 	blob->discriminator_type.offset = 0;
-	
+
 	members = g_list_copy (union_->members);
 
 	g_ir_node_build_members (&members, G_IR_NODE_FIELD, &blob->n_fields,
@@ -1993,7 +2024,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	    for (l = union_->discriminators; l; l = l->next)
 	      {
 		GIrNode *member = (GIrNode *)l->data;
-		
+
 		g_ir_node_build_typelib (member, node, build, offset, offset2);
 	      }
 	  }
@@ -2006,13 +2037,13 @@ g_ir_node_build_typelib (GIrNode         *node,
 	EnumBlob *blob = (EnumBlob *)&data[*offset];
 	GIrNodeEnum *enum_ = (GIrNodeEnum *)node;
 
-	*offset += sizeof (EnumBlob); 
+	*offset += sizeof (EnumBlob);
 
 	if (node->type == G_IR_NODE_ENUM)
 	  blob->blob_type = BLOB_TYPE_ENUM;
 	else
 	  blob->blob_type = BLOB_TYPE_FLAGS;
-	  
+
 	blob->deprecated = enum_->deprecated;
 	blob->reserved = 0;
 	blob->storage_type = enum_->storage_type;
@@ -2042,7 +2073,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	  }
       }
       break;
-      
+
     case G_IR_NODE_OBJECT:
       {
 	ObjectBlob *blob = (ObjectBlob *)&data[*offset];
@@ -2072,7 +2103,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->n_signals = 0;
 	blob->n_vfuncs = 0;
 	blob->n_constants = 0;
-	
+
 	*offset += sizeof(ObjectBlob);
 	for (l = object->interfaces; l; l = l->next)
 	  {
@@ -2080,7 +2111,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	    *(guint16*)&data[*offset] = find_entry (module, modules, (gchar *)l->data);
 	    *offset += 2;
 	  }
-	
+
 	members = g_list_copy (object->members);
 
 	*offset = ALIGN_VALUE (*offset, 4);
@@ -2135,7 +2166,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	blob->n_signals = 0;
 	blob->n_vfuncs = 0;
 	blob->n_constants = 0;
-	
+
 	*offset += sizeof (InterfaceBlob);
 	for (l = iface->prerequisites; l; l = l->next)
 	  {
@@ -2143,7 +2174,7 @@ g_ir_node_build_typelib (GIrNode         *node,
 	    *(guint16*)&data[*offset] = find_entry (module, modules, (gchar *)l->data);
 	    *offset += 2;
 	  }
-	
+
 	members = g_list_copy (iface->members);
 
 	*offset = ALIGN_VALUE (*offset, 4);
@@ -2249,11 +2280,11 @@ g_ir_node_build_typelib (GIrNode         *node,
 	    break;
 	  case GI_TYPE_TAG_INT64:
 	    blob->size = 8;
-	    *(gint64*)&data[blob->offset] = (gint64) parse_int_value (constant->value);
+	    DO_ALIGNED_COPY(&data[blob->offset], parse_int_value (constant->value), gint64);
 	    break;
 	  case GI_TYPE_TAG_UINT64:
 	    blob->size = 8;
-	    *(guint64*)&data[blob->offset] = (guint64) parse_uint_value (constant->value);
+	    DO_ALIGNED_COPY(&data[blob->offset], parse_uint_value (constant->value), guint64);
 	    break;
 	  case GI_TYPE_TAG_SHORT:
 	    blob->size = sizeof (gshort);
@@ -2274,21 +2305,21 @@ g_ir_node_build_typelib (GIrNode         *node,
 	  case GI_TYPE_TAG_SSIZE: /* FIXME */
 	  case GI_TYPE_TAG_LONG:
 	    blob->size = sizeof (glong);
-	    *(glong*)&data[blob->offset] = (glong) parse_int_value (constant->value);
+	    DO_ALIGNED_COPY(&data[blob->offset], parse_int_value (constant->value), glong);
 	    break;
 	  case GI_TYPE_TAG_SIZE: /* FIXME */
-	  case GI_TYPE_TAG_TIME_T: 
+	  case GI_TYPE_TAG_TIME_T:
 	  case GI_TYPE_TAG_ULONG:
 	    blob->size = sizeof (gulong);
-	    *(gulong*)&data[blob->offset] = (gulong) parse_uint_value (constant->value);
+	    DO_ALIGNED_COPY(&data[blob->offset], parse_uint_value (constant->value), gulong);
 	    break;
 	  case GI_TYPE_TAG_FLOAT:
 	    blob->size = sizeof (gfloat);
-	    *(gfloat*)&data[blob->offset] = (gfloat) parse_float_value (constant->value);
+	    DO_ALIGNED_COPY(&data[blob->offset], parse_float_value (constant->value), gfloat);
 	    break;
 	  case GI_TYPE_TAG_DOUBLE:
 	    blob->size = sizeof (gdouble);
-	    *(gdouble*)&data[blob->offset] = (gdouble) parse_float_value (constant->value);
+	    DO_ALIGNED_COPY(&data[blob->offset], parse_float_value (constant->value), gdouble);
 	    break;
 	  case GI_TYPE_TAG_UTF8:
 	  case GI_TYPE_TAG_FILENAME:
@@ -2297,14 +2328,14 @@ g_ir_node_build_typelib (GIrNode         *node,
 	    break;
 	  }
 	*offset2 += ALIGN_VALUE (blob->size, 4);
-	
+
 	g_ir_node_build_typelib ((GIrNode *)constant->type, node, build, &pos, offset2);
       }
       break;
     default:
       g_assert_not_reached ();
     }
-  
+
   g_debug ("node %s%s%s%p type '%s', offset %d -> %d, offset2 %d -> %d",
 	   node->name ? "'" : "",
 	   node->name ? node->name : "",
@@ -2318,12 +2349,12 @@ g_ir_node_build_typelib (GIrNode         *node,
 }
 
 /* if str is already in the pool, return previous location, otherwise write str
- * to the typelib at offset, put it in the pool and update offset. If the 
+ * to the typelib at offset, put it in the pool and update offset. If the
  * typelib is not large enough to hold the string, reallocate it.
  */
-guint32 
+guint32
 write_string (const gchar *str,
-	      GHashTable  *strings, 
+	      GHashTable  *strings,
 	      guchar      *data,
 	      guint32     *offset)
 {
@@ -2334,7 +2365,7 @@ write_string (const gchar *str,
   string_size += strlen (str);
 
   value = g_hash_table_lookup (strings, str);
-  
+
   if (value)
     return GPOINTER_TO_UINT (value);
 
@@ -2347,7 +2378,7 @@ write_string (const gchar *str,
   *offset = ALIGN_VALUE (start + strlen (str) + 1, 4);
 
   strcpy ((gchar*)&data[start], str);
-  
+
   return start;
 }
 
