@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "girffi.h"
+#include "girffi-private.h"
 #include "girnode.h"
 
 /* The C standard specifies that an enumeration can be any char or any signed
@@ -67,7 +67,7 @@ static void
 compute_enum_storage_type (GIrNodeEnum *enum_node)
 {
   GList *l;
-  guint32 max_value = 0;
+  gint32 max_value = 0;
   int width;
 
   if (enum_node->storage_type != GI_TYPE_TAG_VOID) /* already done */
@@ -77,7 +77,7 @@ compute_enum_storage_type (GIrNodeEnum *enum_node)
     {
       GIrNodeValue *value = l->data;
       if (value->value > max_value)
-	max_value = value->value;
+        max_value = value->value;
     }
 
   if (max_value < 128)
@@ -156,7 +156,7 @@ get_interface_size_alignment (GIrNodeType *type,
 
   if (!g_ir_find_node (module, modules, type->interface, &iface, &iface_module))
     {
-      g_warning ("Can't resolve type '%s' for %s", type->interface, who);
+      g_ir_module_fatal (module, 0, "Can't resolve type '%s' for %s", type->interface, who);
       *size = -1;
       *alignment = -1;
       return FALSE;
@@ -235,7 +235,7 @@ get_type_size_alignment (GIrNodeType *type,
   if (type->tag == GI_TYPE_TAG_ARRAY)
     {
       gint elt_size, elt_alignment;
-          
+
       if (!type->has_size
           || !get_type_size_alignment(type->parameter_type1, module, modules,
                                       &elt_size, &elt_alignment, who))
@@ -244,10 +244,10 @@ get_type_size_alignment (GIrNodeType *type,
           *alignment = -1;
           return FALSE;
         }
-          
+
       *size = type->size * elt_size;
       *alignment = elt_alignment;
-          
+
       return TRUE;
     }
   else if (type->is_pointer)
@@ -262,7 +262,7 @@ get_type_size_alignment (GIrNodeType *type,
 	}
       else
 	{
-	  type_ffi = g_ir_ffi_get_ffi_type (type->tag);
+	  type_ffi = g_ir_ffi_get_ffi_type (type->tag, type->is_pointer);
 
 	  if (type_ffi == &ffi_type_void)
 	    {
@@ -300,7 +300,7 @@ get_field_size_alignment (GIrNodeField *field,
 {
   gchar *who;
   gboolean success;
-  
+
   who = g_strdup_printf ("field %s.%s.%s", module->name, parent_node->name, ((GIrNode *)field)->name);
 
   if (field->callback)
