@@ -1,4 +1,4 @@
-/* GObject introspection: typelib validation, auxiliary functions 
+/* GObject introspection: typelib validation, auxiliary functions
  * related to the binary typelib format
  *
  * Copyright (C) 2005 Matthias Clasen
@@ -26,6 +26,7 @@
 
 #include "config.h"
 #include "gtypelib.h"
+#include "glib-compat.h"
 
 typedef struct {
   GTypelib *typelib;
@@ -45,7 +46,7 @@ static void
 pop_context (ValidateContext *ctx)
 {
   g_assert (ctx->context_stack != NULL);
-  ctx->context_stack = g_slist_delete_link (ctx->context_stack, 
+  ctx->context_stack = g_slist_delete_link (ctx->context_stack,
 					    ctx->context_stack);
 }
 
@@ -68,7 +69,7 @@ get_dir_entry_checked (GTypelib *typelib,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_BLOB,
 		   "Invalid directory index %d", index);
-      return FALSE;	        
+      return FALSE;
     }
 
   offset = header->directory + (index - 1) * header->entry_blob_size;
@@ -116,13 +117,13 @@ get_type_blob (GTypelib *typelib,
       return FALSE;
     }
 
-  if (simple->reserved == 0 && simple->reserved2 == 0)
+  if (simple->flags.reserved == 0 && simple->flags.reserved2 == 0)
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID,
 		   "Expected non-basic type but got %d",
-		   simple->tag);
+		   simple->flags.tag);
       return FALSE;
     }
 
@@ -138,7 +139,7 @@ g_typelib_get_dir_entry (GTypelib *typelib,
   return (DirEntry *)&typelib->data[header->directory + (index - 1) * header->entry_blob_size];
 }
 
-void    
+void
 g_typelib_check_sanity (void)
 {
   /* Check that struct layout is as we expect */
@@ -160,7 +161,7 @@ g_typelib_check_sanity (void)
    *
    * Everything else in the code however should be using sizeof().
    */
-  
+
   CHECK_SIZE (Header, 112);
   CHECK_SIZE (DirEntry, 12);
   CHECK_SIZE (SimpleTypeBlob, 4);
@@ -236,7 +237,7 @@ validate_name (GTypelib   *typelib,
   if (!name)
     return FALSE;
 
-  if (!memchr (name, '\0', MAX_NAME_LEN)) 
+  if (!memchr (name, '\0', MAX_NAME_LEN))
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -245,21 +246,21 @@ validate_name (GTypelib   *typelib,
 		   msg, name);
       return FALSE;
     }
-  
-  if (strspn (name, G_CSET_a_2_z G_CSET_A_2_Z G_CSET_DIGITS "-_") < strlen (name)) 
+
+  if (strspn (name, G_CSET_a_2_z G_CSET_A_2_Z G_CSET_DIGITS "-_") < strlen (name))
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID,
-		   "The %s is contains invalid characters: %s",
+		   "The %s contains invalid characters: '%s'",
 		   msg, name);
       return FALSE;
     }
-  
+
   return TRUE;
 }
 
-static gboolean 
+static gboolean
 validate_header (ValidateContext  *ctx,
 		 GError          **error)
 {
@@ -284,7 +285,7 @@ validate_header (ValidateContext  *ctx,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
 		   "Magic string not found");
       return FALSE;
-      
+
     }
 
   if (header->major_version != 2 || header->minor_version != 0)
@@ -294,7 +295,7 @@ validate_header (ValidateContext  *ctx,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
 		   "Version mismatch");
       return FALSE;
-      
+
     }
 
   if (header->n_entries < header->n_local_entries)
@@ -303,7 +304,7 @@ validate_header (ValidateContext  *ctx,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
 		   "Inconsistent entry counts");
-      return FALSE; 
+      return FALSE;
     }
 
   if (header->size != typelib->len)
@@ -312,12 +313,12 @@ validate_header (ValidateContext  *ctx,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
 		   "Typelib size mismatch");
-      return FALSE; 
+      return FALSE;
     }
-  
+
   /* This is a sanity check for a specific typelib; it
    * prevents us from loading an incompatible typelib.
-   * 
+   *
    * The hardcoded checks in g_typelib_check_sanity to
    * protect against inadvertent or buggy changes to the typelib format
    * itself.
@@ -346,7 +347,7 @@ validate_header (ValidateContext  *ctx,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
 		   "Blob size mismatch");
-      return FALSE; 
+      return FALSE;
     }
 
   if (!is_aligned (header->directory))
@@ -355,7 +356,7 @@ validate_header (ValidateContext  *ctx,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
 		   "Misaligned directory");
-      return FALSE; 
+      return FALSE;
     }
 
   if (!is_aligned (header->attributes))
@@ -364,7 +365,7 @@ validate_header (ValidateContext  *ctx,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
 		   "Misaligned attributes");
-      return FALSE; 
+      return FALSE;
     }
 
   if (header->attributes == 0 && header->n_attributes > 0)
@@ -373,11 +374,11 @@ validate_header (ValidateContext  *ctx,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_HEADER,
 		   "Wrong number of attributes");
-      return FALSE; 
+      return FALSE;
     }
 
   if (!validate_name (typelib, "namespace", typelib->data, header->namespace, error))
-    return FALSE; 
+    return FALSE;
 
   return TRUE;
 }
@@ -405,7 +406,7 @@ validate_array_type_blob (GTypelib     *typelib,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_BLOB,
 		   "Pointer type exected for tag %d", blob->tag);
-      return FALSE;	  
+      return FALSE;
     }
 
   /* FIXME validate length */
@@ -459,18 +460,18 @@ validate_param_type_blob (GTypelib     *typelib,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_BLOB,
 		   "Pointer type exected for tag %d", blob->tag);
-      return FALSE;	  
+      return FALSE;
     }
-  
+
   if (blob->n_types != n_params)
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_BLOB,
 		   "Parameter type number mismatch");
-      return FALSE;	        
+      return FALSE;
     }
-  
+
   for (i = 0; i < n_params; i++)
     {
       if (!validate_type_blob (typelib,
@@ -505,9 +506,9 @@ validate_error_type_blob (GTypelib     *typelib,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_BLOB,
 		   "Pointer type exected for tag %d", blob->tag);
-      return FALSE;	  
+      return FALSE;
     }
-  
+
   for (i = 0; i < blob->n_domains; i++)
     {
       if (blob->domains[i] == 0 || blob->domains[i] > header->n_entries)
@@ -516,7 +517,7 @@ validate_error_type_blob (GTypelib     *typelib,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Invalid directory index %d", blob->domains[i]);
-	  return FALSE;	        
+	  return FALSE;
 	}
 
       entry = g_typelib_get_dir_entry (typelib, blob->domains[i]);
@@ -528,7 +529,7 @@ validate_error_type_blob (GTypelib     *typelib,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Wrong blob type");
-	  return FALSE;	        
+	  return FALSE;
 	}
     }
 
@@ -544,13 +545,13 @@ validate_type_blob (GTypelib     *typelib,
 {
   SimpleTypeBlob *simple;
   InterfaceTypeBlob *iface;
-  
+
   simple = (SimpleTypeBlob *)&typelib->data[offset];
 
-  if (simple->reserved == 0 && 
-      simple->reserved2 == 0)
+  if (simple->flags.reserved == 0 &&
+      simple->flags.reserved2 == 0)
     {
-      if (simple->tag >= GI_TYPE_TAG_ARRAY)
+      if (simple->flags.tag >= GI_TYPE_TAG_ARRAY)
 	{
 	  g_set_error (error,
 		       G_TYPELIB_ERROR,
@@ -558,15 +559,15 @@ validate_type_blob (GTypelib     *typelib,
 		       "Wrong tag in simple type");
 	  return FALSE;
 	}
-      
-      if (simple->tag >= GI_TYPE_TAG_UTF8 &&
-	  !simple->pointer)
+
+      if (simple->flags.tag >= GI_TYPE_TAG_UTF8 &&
+	  !simple->flags.pointer)
 	{
 	  g_set_error (error,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
-		       "Pointer type exected for tag %d", simple->tag);
-	  return FALSE;	  
+		       "Pointer type exected for tag %d", simple->flags.tag);
+	  return FALSE;
 	}
 
       return TRUE;
@@ -577,28 +578,28 @@ validate_type_blob (GTypelib     *typelib,
   switch (iface->tag)
     {
     case GI_TYPE_TAG_ARRAY:
-      if (!validate_array_type_blob (typelib, simple->offset, 
+      if (!validate_array_type_blob (typelib, simple->offset,
 				     signature_offset, return_type, error))
 	return FALSE;
       break;
     case GI_TYPE_TAG_INTERFACE:
-      if (!validate_iface_type_blob (typelib, simple->offset, 
+      if (!validate_iface_type_blob (typelib, simple->offset,
 				     signature_offset, return_type, error))
 	return FALSE;
       break;
     case GI_TYPE_TAG_GLIST:
     case GI_TYPE_TAG_GSLIST:
-      if (!validate_param_type_blob (typelib, simple->offset, 
+      if (!validate_param_type_blob (typelib, simple->offset,
 				     signature_offset, return_type, 1, error))
 	return FALSE;
       break;
     case GI_TYPE_TAG_GHASH:
-      if (!validate_param_type_blob (typelib, simple->offset, 
+      if (!validate_param_type_blob (typelib, simple->offset,
 				     signature_offset, return_type, 2, error))
 	return FALSE;
       break;
     case GI_TYPE_TAG_ERROR:
-      if (!validate_error_type_blob (typelib, simple->offset, 
+      if (!validate_error_type_blob (typelib, simple->offset,
 				     signature_offset, return_type, error))
 	return FALSE;
       break;
@@ -633,10 +634,10 @@ validate_arg_blob (GTypelib     *typelib,
   blob = (ArgBlob*) &typelib->data[offset];
 
   if (!validate_name (typelib, "argument", typelib->data, blob->name, error))
-    return FALSE; 
- 
-  if (!validate_type_blob (typelib, 
-			   offset + G_STRUCT_OFFSET (ArgBlob, arg_type), 
+    return FALSE;
+
+  if (!validate_type_blob (typelib,
+			   offset + G_STRUCT_OFFSET (ArgBlob, arg_type),
 			   signature_offset, FALSE, error))
     return FALSE;
 
@@ -692,18 +693,18 @@ validate_signature_blob (GTypelib     *typelib,
 
   if (blob->return_type.offset != 0)
     {
-      if (!validate_type_blob (typelib, 
-			       offset + G_STRUCT_OFFSET (SignatureBlob, return_type), 
+      if (!validate_type_blob (typelib,
+			       offset + G_STRUCT_OFFSET (SignatureBlob, return_type),
 			       offset, TRUE, error))
 	return FALSE;
     }
 
   for (i = 0; i < blob->n_arguments; i++)
     {
-      if (!validate_arg_blob (typelib, 
-			      offset + sizeof (SignatureBlob) + 
-			      i * sizeof (ArgBlob), 
-			      offset, 
+      if (!validate_arg_blob (typelib,
+			      offset + sizeof (SignatureBlob) +
+			      i * sizeof (ArgBlob),
+			      offset,
 			      error))
 	return FALSE;
     }
@@ -745,12 +746,12 @@ validate_function_blob (ValidateContext *ctx,
     }
 
   if (!validate_name (typelib, "function", typelib->data, blob->name, error))
-    return FALSE; 
+    return FALSE;
 
   push_context (ctx, get_string_nofail (typelib, blob->name));
-  
+
   if (!validate_name (typelib, "function symbol", typelib->data, blob->symbol, error))
-    return FALSE; 
+    return FALSE;
 
   switch (container_type)
     {
@@ -764,7 +765,7 @@ validate_function_blob (ValidateContext *ctx,
     default:
       is_method = FALSE;
     }
-  
+
   if (blob->constructor)
     {
       switch (container_type)
@@ -819,7 +820,7 @@ validate_function_blob (ValidateContext *ctx,
 
   sigblob = (SignatureBlob*) &typelib->data[blob->signature];
 
-  if (blob->constructor) 
+  if (blob->constructor)
     {
       SimpleTypeBlob *simple = return_type_from_signature (typelib,
 							   blob->signature,
@@ -876,15 +877,15 @@ validate_callback_blob (ValidateContext *ctx,
     }
 
   if (!validate_name (typelib, "callback", typelib->data, blob->name, error))
-    return FALSE; 
+    return FALSE;
 
   push_context (ctx, get_string_nofail (typelib, blob->name));
-  
+
   if (!validate_signature_blob (typelib, blob->signature, error))
     return FALSE;
 
   pop_context (ctx);
-	
+
   return TRUE;
 }
 
@@ -949,9 +950,9 @@ validate_constant_blob (GTypelib     *typelib,
     }
 
   if (!validate_name (typelib, "constant", typelib->data, blob->name, error))
-    return FALSE; 
-  
-  if (!validate_type_blob (typelib, offset + G_STRUCT_OFFSET (ConstantBlob, type), 
+    return FALSE;
+
+  if (!validate_type_blob (typelib, offset + G_STRUCT_OFFSET (ConstantBlob, type),
 			   0, FALSE, error))
     return FALSE;
 
@@ -963,11 +964,11 @@ validate_constant_blob (GTypelib     *typelib,
 		   "Misaligned constant value");
       return FALSE;
     }
-  
+
   type = (SimpleTypeBlob *)&typelib->data[offset + G_STRUCT_OFFSET (ConstantBlob, type)];
-  if (type->reserved == 0 && type->reserved2 == 0)
+  if (type->flags.reserved == 0 && type->flags.reserved2 == 0)
     {
-      if (type->tag == 0)
+      if (type->flags.tag == 0)
 	{
 	  g_set_error (error,
 		       G_TYPELIB_ERROR,
@@ -976,8 +977,8 @@ validate_constant_blob (GTypelib     *typelib,
 	  return FALSE;
 	}
 
-      if (value_size[type->tag] != 0 &&
-	  blob->size != value_size[type->tag])
+      if (value_size[type->flags.tag] != 0 &&
+	  blob->size != value_size[type->flags.tag])
 	{
 	  g_set_error (error,
 		       G_TYPELIB_ERROR,
@@ -1010,16 +1011,18 @@ validate_value_blob (GTypelib     *typelib,
   blob = (ValueBlob*) &typelib->data[offset];
 
   if (!validate_name (typelib, "value", typelib->data, blob->name, error))
-    return FALSE; 
-  
+    return FALSE;
+
   return TRUE;
 }
 
 static gboolean
-validate_field_blob (GTypelib     *typelib,
+validate_field_blob (ValidateContext *ctx,
 		     guint32        offset,
 		     GError       **error)
 {
+  GTypelib *typelib = ctx->typelib;
+  Header *header = (Header *)typelib->data;
   FieldBlob *blob;
 
   if (typelib->len < offset + sizeof (FieldBlob))
@@ -1032,13 +1035,18 @@ validate_field_blob (GTypelib     *typelib,
     }
 
   blob = (FieldBlob*) &typelib->data[offset];
-  
+
   if (!validate_name (typelib, "field", typelib->data, blob->name, error))
-    return FALSE; 
-  
-  if (!validate_type_blob (typelib,
-			   offset + G_STRUCT_OFFSET (FieldBlob, type), 
-			   0, FALSE, error))
+    return FALSE;
+
+  if (blob->has_embedded_type)
+    {
+      if (!validate_callback_blob (ctx, offset + header->field_blob_size, error))
+        return FALSE;
+    }
+  else if (!validate_type_blob (typelib,
+			        offset + G_STRUCT_OFFSET (FieldBlob, type),
+			        0, FALSE, error))
     return FALSE;
 
   return TRUE;
@@ -1061,12 +1069,12 @@ validate_property_blob (GTypelib     *typelib,
     }
 
   blob = (PropertyBlob*) &typelib->data[offset];
-  
+
   if (!validate_name (typelib, "property", typelib->data, blob->name, error))
-    return FALSE; 
-    
+    return FALSE;
+
   if (!validate_type_blob (typelib,
-			   offset + G_STRUCT_OFFSET (PropertyBlob, type), 
+			   offset + G_STRUCT_OFFSET (PropertyBlob, type),
 			   0, FALSE, error))
     return FALSE;
 
@@ -1094,17 +1102,17 @@ validate_signal_blob (GTypelib     *typelib,
   blob = (SignalBlob*) &typelib->data[offset];
 
   if (!validate_name (typelib, "signal", typelib->data, blob->name, error))
-    return FALSE; 
-  
-  if ((blob->run_first != 0) + 
-      (blob->run_last != 0) + 
+    return FALSE;
+
+  if ((blob->run_first != 0) +
+      (blob->run_last != 0) +
       (blob->run_cleanup != 0) != 1)
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_BLOB,
 		   "Invalid signal run flags");
-      return FALSE; 
+      return FALSE;
     }
 
   if (blob->has_class_closure)
@@ -1114,15 +1122,15 @@ validate_signal_blob (GTypelib     *typelib,
 	  ObjectBlob *object;
 
 	  object = (ObjectBlob*)&typelib->data[container_offset];
-	  
+
 	  n_signals = object->n_signals;
 	}
       else
 	{
 	  InterfaceBlob *iface;
-	  
+
 	  iface = (InterfaceBlob*)&typelib->data[container_offset];
-	  
+
 	  n_signals = iface->n_signals;
 	}
 
@@ -1132,13 +1140,13 @@ validate_signal_blob (GTypelib     *typelib,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Invalid class closure index");
-	  return FALSE; 
+	  return FALSE;
 	}
     }
 
   if (!validate_signature_blob (typelib, blob->signature, error))
     return FALSE;
-  
+
   return TRUE;
 }
 
@@ -1163,8 +1171,8 @@ validate_vfunc_blob (GTypelib     *typelib,
   blob = (VFuncBlob*) &typelib->data[offset];
 
   if (!validate_name (typelib, "vfunc", typelib->data, blob->name, error))
-    return FALSE; 
-  
+    return FALSE;
+
   if (blob->class_closure)
     {
       if (((CommonBlob*)&typelib->data[container_offset])->blob_type == BLOB_TYPE_OBJECT)
@@ -1172,15 +1180,15 @@ validate_vfunc_blob (GTypelib     *typelib,
 	  ObjectBlob *object;
 
 	  object = (ObjectBlob*)&typelib->data[container_offset];
-	  
+
 	  n_vfuncs = object->n_vfuncs;
 	}
       else
 	{
 	  InterfaceBlob *iface;
-	  
+
 	  iface = (InterfaceBlob*)&typelib->data[container_offset];
-	  
+
 	  n_vfuncs = iface->n_vfuncs;
 	}
 
@@ -1190,13 +1198,13 @@ validate_vfunc_blob (GTypelib     *typelib,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Invalid class closure index");
-	  return FALSE; 
+	  return FALSE;
 	}
     }
 
   if (!validate_signature_blob (typelib, blob->signature, error))
     return FALSE;
-  
+
   return TRUE;
 }
 
@@ -1209,6 +1217,7 @@ validate_struct_blob (ValidateContext *ctx,
   GTypelib *typelib = ctx->typelib;
   StructBlob *blob;
   gint i;
+  guint32 field_offset;
 
   if (typelib->len < offset + sizeof (StructBlob))
     {
@@ -1231,17 +1240,17 @@ validate_struct_blob (ValidateContext *ctx,
     }
 
   if (!validate_name (typelib, "struct", typelib->data, blob->name, error))
-    return FALSE; 
+    return FALSE;
 
   push_context (ctx, get_string_nofail (typelib, blob->name));
-  
+
   if (!blob->unregistered)
     {
       if (!validate_name (typelib, "boxed", typelib->data, blob->gtype_name, error))
-	return FALSE; 
+	return FALSE;
 
       if (!validate_name (typelib, "boxed", typelib->data, blob->gtype_init, error))
-	return FALSE; 
+	return FALSE;
     }
   else
     {
@@ -1251,11 +1260,11 @@ validate_struct_blob (ValidateContext *ctx,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Gtype data in struct");
-	  return FALSE; 
+	  return FALSE;
 	}
     }
 
-  if (typelib->len < offset + sizeof (StructBlob) + 
+  if (typelib->len < offset + sizeof (StructBlob) +
             blob->n_fields * sizeof (FieldBlob) +
             blob->n_methods * sizeof (FunctionBlob))
     {
@@ -1266,21 +1275,26 @@ validate_struct_blob (ValidateContext *ctx,
       return FALSE;
     }
 
+  field_offset = offset + sizeof (StructBlob);
   for (i = 0; i < blob->n_fields; i++)
     {
-      if (!validate_field_blob (typelib, 
-				offset + sizeof (StructBlob) + 
-				i * sizeof (FieldBlob), 
+      FieldBlob *blob = (FieldBlob*) &typelib->data[field_offset];
+
+      if (!validate_field_blob (ctx,
+				field_offset,
 				error))
 	return FALSE;
+
+      field_offset += sizeof (FieldBlob);
+      if (blob->has_embedded_type)
+        field_offset += sizeof (CallbackBlob);
     }
 
   for (i = 0; i < blob->n_methods; i++)
     {
-      if (!validate_function_blob (ctx, 
-				   offset + sizeof (StructBlob) + 
-				   blob->n_fields * sizeof (FieldBlob) + 
-				   i * sizeof (FunctionBlob), 
+      if (!validate_function_blob (ctx,
+				   field_offset +
+				   i * sizeof (FunctionBlob),
 				   blob_type,
 				   error))
 	return FALSE;
@@ -1320,14 +1334,14 @@ validate_enum_blob (ValidateContext *ctx,
 		   "Wrong blob type");
       return FALSE;
     }
-  
+
   if (!blob->unregistered)
     {
       if (!validate_name (typelib, "enum", typelib->data, blob->gtype_name, error))
-	return FALSE; 
+	return FALSE;
 
       if (!validate_name (typelib, "enum", typelib->data, blob->gtype_init, error))
-	return FALSE; 
+	return FALSE;
     }
   else
     {
@@ -1337,14 +1351,14 @@ validate_enum_blob (ValidateContext *ctx,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Gtype data in unregistered enum");
-	  return FALSE; 
+	  return FALSE;
 	}
     }
 
   if (!validate_name (typelib, "enum", typelib->data, blob->name, error))
-    return FALSE; 
-  
-  if (typelib->len < offset + sizeof (EnumBlob) + 
+    return FALSE;
+
+  if (typelib->len < offset + sizeof (EnumBlob) +
       blob->n_values * sizeof (ValueBlob))
     {
       g_set_error (error,
@@ -1355,21 +1369,21 @@ validate_enum_blob (ValidateContext *ctx,
     }
 
   push_context (ctx, get_string_nofail (typelib, blob->name));
-  
+
   for (i = 0; i < blob->n_values; i++)
     {
-      if (!validate_value_blob (typelib, 
-				offset + sizeof (EnumBlob) + 
-				i * sizeof (ValueBlob), 
+      if (!validate_value_blob (typelib,
+				offset + sizeof (EnumBlob) +
+				i * sizeof (ValueBlob),
 				error))
 	return FALSE;
 
 #if 0
-      v1 = (ValueBlob *)&typelib->data[offset + sizeof (EnumBlob) + 
+      v1 = (ValueBlob *)&typelib->data[offset + sizeof (EnumBlob) +
                                         i * sizeof (ValueBlob)];
-      for (j = 0; j < i; j++) 
+      for (j = 0; j < i; j++)
 	{
-	  v2 = (ValueBlob *)&typelib->data[offset + sizeof (EnumBlob) + 
+	  v2 = (ValueBlob *)&typelib->data[offset + sizeof (EnumBlob) +
                                             j * sizeof (ValueBlob)];
 
 	  if (v1->value == v2->value)
@@ -1383,11 +1397,11 @@ validate_enum_blob (ValidateContext *ctx,
 	      return FALSE;
 	    }
 	}
-#endif      
+#endif
     }
 
   pop_context (ctx);
-  
+
   return TRUE;
 }
 
@@ -1423,15 +1437,15 @@ validate_object_blob (ValidateContext *ctx,
 		   "Wrong blob type");
       return FALSE;
     }
-  
+
   if (!validate_name (typelib, "object", typelib->data, blob->gtype_name, error))
-    return FALSE; 
-  
+    return FALSE;
+
   if (!validate_name (typelib, "object", typelib->data, blob->gtype_init, error))
-    return FALSE; 
-  
+    return FALSE;
+
   if (!validate_name (typelib, "object", typelib->data, blob->name, error))
-    return FALSE; 
+    return FALSE;
 
   if (blob->parent > header->n_entries)
     {
@@ -1439,7 +1453,7 @@ validate_object_blob (ValidateContext *ctx,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_BLOB,
 		   "Invalid parent index");
-      return FALSE; 
+      return FALSE;
     }
 
   if (blob->parent != 0)
@@ -1456,10 +1470,10 @@ validate_object_blob (ValidateContext *ctx,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Parent not object");
-	  return FALSE; 
+	  return FALSE;
 	}
     }
-  
+
   if (blob->gtype_struct != 0)
     {
       DirEntry *entry;
@@ -1473,11 +1487,11 @@ validate_object_blob (ValidateContext *ctx,
                        G_TYPELIB_ERROR,
                        G_TYPELIB_ERROR_INVALID_BLOB,
                        "Class struct invalid type or not local");
-          return FALSE; 
+          return FALSE;
         }
-    }  
-  
-  if (typelib->len < offset + sizeof (ObjectBlob) + 
+    }
+
+  if (typelib->len < offset + sizeof (ObjectBlob) +
             (blob->n_interfaces + blob->n_interfaces % 2) * 2 +
             blob->n_fields * sizeof (FieldBlob) +
             blob->n_properties * sizeof (PropertyBlob) +
@@ -1485,7 +1499,7 @@ validate_object_blob (ValidateContext *ctx,
             blob->n_signals * sizeof (SignalBlob) +
             blob->n_vfuncs * sizeof (VFuncBlob) +
             blob->n_constants * sizeof (ConstantBlob))
-     
+
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -1508,9 +1522,9 @@ validate_object_blob (ValidateContext *ctx,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Invalid interface index");
-	  return FALSE; 
+	  return FALSE;
 	}
-      
+
       entry = get_dir_entry_checked (typelib, iface, error);
       if (!entry)
         return FALSE;
@@ -1522,17 +1536,17 @@ validate_object_blob (ValidateContext *ctx,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Not an interface");
-	  return FALSE; 
+	  return FALSE;
 	}
     }
 
   offset2 += 2 * (blob->n_interfaces %2);
 
   push_context (ctx, get_string_nofail (typelib, blob->name));
-  
+
   for (i = 0; i < blob->n_fields; i++, offset2 += sizeof (FieldBlob))
     {
-      if (!validate_field_blob (typelib, offset2, error))
+      if (!validate_field_blob (ctx, offset2, error))
 	return FALSE;
     }
 
@@ -1581,7 +1595,7 @@ validate_interface_blob (ValidateContext *ctx,
   InterfaceBlob *blob;
   gint i;
   guint32 offset2;
-  
+
   header = (Header *)typelib->data;
 
   if (typelib->len < offset + sizeof (InterfaceBlob))
@@ -1603,24 +1617,24 @@ validate_interface_blob (ValidateContext *ctx,
 		   "Wrong blob type; expected interface, got %d", blob->blob_type);
       return FALSE;
     }
-  
+
   if (!validate_name (typelib, "interface", typelib->data, blob->gtype_name, error))
-    return FALSE; 
-  
+    return FALSE;
+
   if (!validate_name (typelib, "interface", typelib->data, blob->gtype_init, error))
-    return FALSE; 
-  
+    return FALSE;
+
   if (!validate_name (typelib, "interface", typelib->data, blob->name, error))
-    return FALSE; 
-  
-  if (typelib->len < offset + sizeof (InterfaceBlob) + 
+    return FALSE;
+
+  if (typelib->len < offset + sizeof (InterfaceBlob) +
             (blob->n_prerequisites + blob->n_prerequisites % 2) * 2 +
             blob->n_properties * sizeof (PropertyBlob) +
             blob->n_methods * sizeof (FunctionBlob) +
             blob->n_signals * sizeof (SignalBlob) +
             blob->n_vfuncs * sizeof (VFuncBlob) +
             blob->n_constants * sizeof (ConstantBlob))
-     
+
     {
       g_set_error (error,
 		   G_TYPELIB_ERROR,
@@ -1643,7 +1657,7 @@ validate_interface_blob (ValidateContext *ctx,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Invalid prerequisite index");
-	  return FALSE; 
+	  return FALSE;
 	}
 
       entry = g_typelib_get_dir_entry (typelib, req);
@@ -1655,14 +1669,14 @@ validate_interface_blob (ValidateContext *ctx,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_BLOB,
 		       "Not an interface or object");
-	  return FALSE; 
+	  return FALSE;
 	}
     }
 
   offset2 += 2 * (blob->n_prerequisites % 2);
 
   push_context (ctx, get_string_nofail (typelib, blob->name));
-  
+
   for (i = 0; i < blob->n_properties; i++, offset2 += sizeof (PropertyBlob))
     {
       if (!validate_property_blob (typelib, offset2, error))
@@ -1674,13 +1688,13 @@ validate_interface_blob (ValidateContext *ctx,
       if (!validate_function_blob (ctx, offset2, BLOB_TYPE_INTERFACE, error))
 	return FALSE;
     }
-  
+
   for (i = 0; i < blob->n_signals; i++, offset2 += sizeof (SignalBlob))
     {
       if (!validate_signal_blob (typelib, offset2, offset, error))
 	return FALSE;
     }
-  
+
   for (i = 0; i < blob->n_vfuncs; i++, offset2 += sizeof (VFuncBlob))
     {
       if (!validate_vfunc_blob (typelib, offset2, offset, error))
@@ -1732,7 +1746,7 @@ validate_blob (ValidateContext *ctx,
     }
 
   common = (CommonBlob*)&typelib->data[offset];
-  
+
   switch (common->blob_type)
     {
     case BLOB_TYPE_FUNCTION:
@@ -1774,7 +1788,7 @@ validate_blob (ValidateContext *ctx,
 	return FALSE;
       break;
     default:
-      g_set_error (error, 
+      g_set_error (error,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID_ENTRY,
 		   "Invalid blob type");
@@ -1784,7 +1798,7 @@ validate_blob (ValidateContext *ctx,
   return TRUE;
 }
 
-static gboolean 
+static gboolean
 validate_directory (ValidateContext   *ctx,
 		    GError            **error)
 {
@@ -1792,7 +1806,7 @@ validate_directory (ValidateContext   *ctx,
   Header *header = (Header *)typelib->data;
   DirEntry *entry;
   gint i;
-  
+
   if (typelib->len < header->directory + header->n_entries * sizeof (DirEntry))
     {
       g_set_error (error,
@@ -1807,8 +1821,8 @@ validate_directory (ValidateContext   *ctx,
       entry = g_typelib_get_dir_entry (typelib, i + 1);
 
       if (!validate_name (typelib, "entry", typelib->data, entry->name, error))
-	return FALSE; 
-      
+	return FALSE;
+
       if ((entry->local && entry->blob_type == BLOB_TYPE_INVALID) ||
 	  entry->blob_type > BLOB_TYPE_UNION)
 	{
@@ -1816,7 +1830,7 @@ validate_directory (ValidateContext   *ctx,
 		       G_TYPELIB_ERROR,
 		       G_TYPELIB_ERROR_INVALID_DIRECTORY,
 		       "Invalid entry type");
-	  return FALSE; 
+	  return FALSE;
 	}
 
       if (i < header->n_local_entries)
@@ -1854,7 +1868,7 @@ validate_directory (ValidateContext   *ctx,
 	    }
 
 	  if (!validate_name (typelib, "namespace", typelib->data, entry->offset, error))
-	    return FALSE; 
+	    return FALSE;
 	}
     }
 
@@ -1874,9 +1888,9 @@ validate_attributes (ValidateContext *ctx,
 		   G_TYPELIB_ERROR,
 		   G_TYPELIB_ERROR_INVALID,
 		   "The buffer is too short");
-      return FALSE;      
+      return FALSE;
     }
-  
+
   return TRUE;
 }
 
@@ -1888,7 +1902,7 @@ prefix_with_context (GError **error,
   GString *str = g_string_new (NULL);
   GSList *link;
   char *buf;
-  
+
   link = ctx->context_stack;
   if (!link)
     {
@@ -1908,7 +1922,7 @@ prefix_with_context (GError **error,
   g_free (buf);
 }
 
-gboolean 
+gboolean
 g_typelib_validate (GTypelib     *typelib,
 		     GError       **error)
 {
@@ -1946,95 +1960,110 @@ g_typelib_error_quark (void)
   return quark;
 }
 
-static inline void
-_g_typelib_init (GTypelib *typelib)
+static void
+_g_typelib_do_dlopen (GTypelib *typelib)
 {
   Header *header;
+  const char *shlib_str;
 
   header = (Header *) typelib->data;
+  /* note that NULL shlib means to open the main app, which is allowed */
   if (header->shared_library)
+    shlib_str = g_typelib_get_string (typelib, header->shared_library);
+  else
+    shlib_str = NULL;
+
+  if (shlib_str != NULL && shlib_str[0] != '\0')
     {
-      const gchar *shlib_str;
-      GModule *app_module = NULL;
+      gchar **shlibs;
+      gint i;
 
-      shlib_str = g_typelib_get_string (typelib, header->shared_library);
-      /* note that NULL shlib means to open the main app, which is allowed */
+      /* shared-library is a comma-separated list of libraries */
+      shlibs = g_strsplit (shlib_str, ",", 0);
 
-      if (shlib_str != NULL)
+       /* We load all passed libs unconditionally as if the same library is loaded
+        * again with g_module_open(), the same file handle will be returned. See bug:
+        * http://bugzilla.gnome.org/show_bug.cgi?id=555294
+        */
+      for (i = 0; shlibs[i]; i++)
         {
-          gchar **shlibs;
-          gint i;
+          GModule *module;
 
-          /* shared-library is a comma-separated list of libraries */
-          shlibs = g_strsplit (shlib_str, ",", 0);
-
-          /* We load all passed libs unconditionally as if the same library is loaded
-           * again with dlopen(), the same file handle will be returned. See bug:
-           * http://bugzilla.gnome.org/show_bug.cgi?id=555294
+          /* Glade's autoconnect feature and OpenGL's extension mechanism
+           * as used by Clutter rely on g_module_open(NULL) to work as a means of
+           * accessing the app's symbols. This keeps us from using
+           * G_MODULE_BIND_LOCAL. BIND_LOCAL may have other issues as well;
+           * in general libraries are not expecting multiple copies of
+           * themselves and are not expecting to be unloaded. So we just
+           * load modules globally for now.
            */
-          for (i = 0; shlibs[i]; i++)
+
+          module = g_module_open (shlibs[i], G_MODULE_BIND_LAZY);
+
+          if (module == NULL)
             {
-              GModule *module;
+              GString *shlib_full = g_string_new (shlibs[i]);
 
-              /* Glade's autoconnect feature and OpenGL's extension mechanism
-               * as used by Clutter rely on dlopen(NULL) to work as a means of
-               * accessing the app's symbols. This keeps us from using
-               * G_MODULE_BIND_LOCAL. BIND_LOCAL may have other issues as well;
-               * in general libraries are not expecting multiple copies of
-               * themselves and are not expecting to be unloaded. So we just
-               * load modules globally for now.
-               */
-
-	      module = g_module_open (shlibs[i], G_MODULE_BIND_LAZY);
-
-	      if (module == NULL)
-	        {
-	          GString *shlib_full = g_string_new (shlibs[i]);
-
-	          /* Prefix with "lib", try both .la and .so */
-	          if (!g_str_has_prefix (shlib_full->str, "lib"))
-	            g_string_prepend (shlib_full, "lib");
-	          g_string_append (shlib_full, ".la");
-	          module = g_module_open (shlib_full->str, G_MODULE_BIND_LAZY);
-	          if (module == NULL)
-	            g_string_overwrite (shlib_full, strlen (shlib_full->str)-2, SHLIB_SUFFIX);
-	          module = g_module_open (shlib_full->str, G_MODULE_BIND_LAZY);
-
-	          g_string_free (shlib_full, TRUE);
-	        }
-
-	      if (module == NULL)
+              /* Prefix with "lib", try both .la and .so */
+              if (!g_str_has_prefix (shlib_full->str, "lib"))
+                g_string_prepend (shlib_full, "lib");
+              g_string_append (shlib_full, ".la");
+              module = g_module_open (shlib_full->str, G_MODULE_BIND_LAZY);
+              if (module == NULL)
                 {
-                  g_warning ("Failed to load shared library '%s' referenced by the typelib: %s",
-                             shlibs[i], g_module_error ());
+                  g_string_overwrite (shlib_full, strlen (shlib_full->str)-2, SHLIB_SUFFIX);
+                  module = g_module_open (shlib_full->str, G_MODULE_BIND_LAZY);
                 }
-              else
-                {
-                  typelib->modules = g_list_append (typelib->modules, module);
-                }
-          }
 
-          g_strfreev (shlibs);
-        }
+              g_string_free (shlib_full, TRUE);
+            }
 
-        /* we should make sure the app_module in the end of list so that
-         * it's last symbol source when loading any symbols from modules.
-         * See comments in g_typelib_symbol */
-        app_module = g_module_open (NULL, G_MODULE_BIND_LAZY);
-        if (app_module)
-          typelib->modules = g_list_append (typelib->modules, app_module);
+          if (module == NULL)
+            {
+              g_warning ("Failed to load shared library '%s' referenced by the typelib: %s",
+                         shlibs[i], g_module_error ());
+            }
+          else
+            {
+              typelib->modules = g_list_append (typelib->modules, module);
+            }
+       }
+
+      g_strfreev (shlibs);
     }
+  else
+    {
+      /* If there's no shared-library entry for this module, assume that
+       * the module is for the application.  Some of the hand-written .gir files
+       * in gobject-introspection don't have shared-library entries, but no one
+       * is really going to be calling g_module_symbol on them either.
+       */
+      GModule *module = g_module_open (NULL, 0);
+      if (module == NULL)
+        g_warning ("gtypelib.c: Failed to g_module_open (NULL): %s", g_module_error ());
+      else
+        typelib->modules = g_list_prepend (typelib->modules, module);
+    }
+}
+
+static inline void
+_g_typelib_ensure_open (GTypelib *typelib)
+{
+  if (typelib->open_attempted)
+    return;
+  typelib->open_attempted = TRUE;
+  _g_typelib_do_dlopen (typelib);
 }
 
 /**
  * g_typelib_new_from_memory:
  * @memory: address of memory chunk containing the typelib
  * @len: length of memory chunk containing the typelib
- * 
+ *
  * Creates a new #GTypelib from a memory location.  The memory block
  * pointed to by @typelib will be automatically g_free()d when the
  * repository is destroyed.
- * 
+ *
  * Return value: the new #GTypelib
  **/
 GTypelib *
@@ -2042,12 +2071,12 @@ g_typelib_new_from_memory (guchar *memory, gsize len)
 {
   GTypelib *meta;
 
-  meta = g_new0 (GTypelib, 1);
+  meta = g_slice_new0 (GTypelib);
   meta->data = memory;
   meta->len = len;
   meta->owns_memory = TRUE;
   meta->modules = NULL;
-  _g_typelib_init (meta);
+
   return meta;
 }
 
@@ -2055,9 +2084,9 @@ g_typelib_new_from_memory (guchar *memory, gsize len)
  * g_typelib_new_from_const_memory:
  * @memory: address of memory chunk containing the typelib
  * @len: length of memory chunk containing the typelib
- * 
+ *
  * Creates a new #GTypelib from a memory location.
- * 
+ *
  * Return value: the new #GTypelib
  **/
 GTypelib *
@@ -2065,21 +2094,21 @@ g_typelib_new_from_const_memory (const guchar *memory, gsize len)
 {
   GTypelib *meta;
 
-  meta = g_new0 (GTypelib, 1);
+  meta = g_slice_new0 (GTypelib);
   meta->data = (guchar *) memory;
   meta->len = len;
   meta->owns_memory = FALSE;
   meta->modules = NULL;
-  _g_typelib_init (meta);
+
   return meta;
 }
 
 /**
  * g_typelib_new_from_mapped_file:
  * @mfile: a #GMappedFile, that will be free'd when the repository is destroyed
- * 
+ *
  * Creates a new #GTypelib from a #GMappedFile.
- * 
+ *
  * Return value: the new #GTypelib
  **/
 GTypelib *
@@ -2087,26 +2116,26 @@ g_typelib_new_from_mapped_file (GMappedFile *mfile)
 {
   GTypelib *meta;
 
-  meta = g_new0 (GTypelib, 1);
+  meta = g_slice_new0 (GTypelib);
   meta->mfile = mfile;
   meta->owns_memory = FALSE;
   meta->data = (guchar *) g_mapped_file_get_contents (mfile);
   meta->len = g_mapped_file_get_length (mfile);
-  _g_typelib_init (meta);
+
   return meta;
 }
 
 /**
  * g_typelib_free:
  * @typelib: a #GTypelib
- * 
+ *
  * Free a #GTypelib.
  **/
 void
 g_typelib_free (GTypelib *typelib)
 {
   if (typelib->mfile)
-    g_mapped_file_free (typelib->mfile);
+    g_mapped_file_unref (typelib->mfile);
   else
     if (typelib->owns_memory)
       g_free (typelib->data);
@@ -2115,7 +2144,7 @@ g_typelib_free (GTypelib *typelib)
       g_list_foreach (typelib->modules, (GFunc) g_module_close, NULL);
       g_list_free (typelib->modules);
     }
-  g_free (typelib);
+  g_slice_free (GTypelib, typelib);
 }
 
 const gchar *
@@ -2139,20 +2168,18 @@ g_typelib_symbol (GTypelib *typelib, const char *symbol_name, gpointer *symbol)
 {
   GList *l;
 
+  _g_typelib_ensure_open (typelib);
+
   /*
-   * We want to be able to add symbols to an app or an auxiliary
-   * library to fill in gaps in an introspected library. However,
-   * normally we would only look for symbols in the main library
-   * (the first items in typelib->modules).
+   * The reason for having multiple modules dates from gir-repository
+   * when it was desired to inject code (accessors, etc.) into an
+   * existing library.  In that situation, the first module listed
+   * will be the custom one, which overrides the main one.  A bit
+   * inefficient, but the problem will go away when gir-repository
+   * does.
    *
-   * A more elaborate solution is probably possible, but as a
-   * simple approach for now, if we fail to find a symbol we look
-   * for it in the global module (the last item in type->modules).
-   *
-   * This would not be very efficient if it happened often, since
-   * we always do the failed lookup above first, but very few
-   * symbols should be outside of the main libraries in
-   * typelib->modules so it doesn't matter.
+   * For modules with no shared library, we dlopen'd the current
+   * process above.
    */
   for (l = typelib->modules; l; l = l->next)
     {
